@@ -6,7 +6,7 @@ from enum import IntEnum
 from importlib import import_module
 from pathlib import Path
 from types import ModuleType
-from typing import Any, Callable, Dict, List, Optional, Tuple, Type, Union
+from typing import TYPE_CHECKING, Any, Callable, Dict, Generator, List, Optional, Tuple, Type, Union
 
 from pydantic import BaseModel, Field, root_validator
 from pydantic.fields import FieldInfo, Undefined
@@ -27,9 +27,11 @@ from protobuf_to_pydantic.grpc_types import (
     RepeatedCompositeContainer,
     RepeatedScalarContainer,
 )
-from protobuf_to_pydantic.types import OneOfTypedDict
 from protobuf_to_pydantic.util import Timedelta, create_pydantic_model
 
+if TYPE_CHECKING:
+    from protobuf_to_pydantic.types import DescFromOptionTypedDict, FieldInfoTypedDict, OneOfTypedDict
+
 type_dict: Dict[int, Type] = {
     FieldDescriptor.TYPE_DOUBLE: float,
     FieldDescriptor.TYPE_FLOAT: float,
@@ -72,6 +74,7 @@ _message_default_factory_dict_by_type_name: Dict[str, Any] = {
 
 
 def check_dict_one_of(desc_dict: dict, key_list: List[str]) -> bool:
+    """Check if the key also appears in the dict"""
     if (
         len(
             [
@@ -87,6 +90,7 @@ def check_dict_one_of(desc_dict: dict, key_list: List[str]) -> bool:
 
 
 def replace_file_name_to_class_name(filename: str) -> str:
+    """Convert the protobuf file name to the class name(PEP-8)"""
     # example_proto/common/single.proto -> Example_protoCommonSingle
     prefix: str = "".join([str(i).title() for i in Path(filename.split(".")[0]).joinpath().parts])
     # Example_protoCommonSingle -> ExampleProtoCommonSingle
@@ -159,6 +163,23 @@ def field_param_dict_handle(field_param_dict: dict, default: Any, default_factor
             field_param_dict["type_"] = field_type(**type_param_dict)
 
 
+class JsonAndDict(dict):
+    @classmethod
+    def __get_validators__(cls) -> Generator[Callable, None, None]:
+        yield cls.validate
+
+    @classmethod
+    def validate(cls, v: Union[str, dict]) -> dict:
+        if isinstance(v, str):
+            try:
+                v = json.loads(v)
+            except json.JSONDecodeError:
+                raise ValueError("JSON string is not valid JSON")
+        elif not isinstance(v, dict):
+            raise ValueError("JSON string is not a dict")
+        return v  # type: ignore[return-value]
+
+
 class MessagePaitModel(BaseModel):
     field: Optional[Type[FieldInfo]] = Field(None)
     enable: bool = Field(True)
@@ -182,7 +203,7 @@ class MessagePaitModel(BaseModel):
     unique_items: Optional[bool] = Field(None)
     multiple_of: Optional[int] = Field(None)
     regex: Optional[str] = Field(None)
-    extra: dict = Field(default_factory=dict)
+    extra: JsonAndDict = Field(default_factory=dict)
     type_: Any = Field(None, alias="type")
     validator: Optional[Dict[str, Any]] = Field(None)
     sub: Optional["MessagePaitModel"] = Field(None)
@@ -255,7 +276,7 @@ class M2P(object):
         message_default_factory_dict_by_type_name: Optional[Dict[str, Any]] = None,
     ):
         proto_file_name = msg.DESCRIPTOR.file.name  # type: ignore
-        message_field_dict: Dict[str, Dict[str, str]] = {}
+        message_field_dict: Dict[str, "DescFromOptionTypedDict"] = {}
 
         if proto_file_name.endswith("empty.proto") or parse_msg_desc_method == "ignore":
             pass
@@ -263,14 +284,14 @@ class M2P(object):
             file_str: str = parse_msg_desc_method
             if not file_str.endswith("/"):
                 file_str += "/"
-            message_field_dict = get_desc_from_proto_file(file_str + proto_file_name)
+            message_field_dict = get_desc_from_proto_file(file_str + proto_file_name, comment_prefix)
         elif inspect.ismodule(parse_msg_desc_method):
             if getattr(parse_msg_desc_method, msg.__name__, None) is not msg:  # type: ignore
                 raise ValueError(f"Not the module corresponding to {msg}")
             pyi_file_name = parse_msg_desc_method.__file__ + "i"  # type: ignore
             if not Path(pyi_file_name).exists():
                 raise RuntimeError(f"Can not found {msg} pyi file")
-            message_field_dict = get_desc_from_pyi_file(pyi_file_name)
+            message_field_dict = get_desc_from_pyi_file(pyi_file_name, comment_prefix)
         elif parse_msg_desc_method == "PGV":
             message_field_dict = get_desc_from_pgv(message=msg)  # type: ignore
         elif parse_msg_desc_method is not None:
@@ -283,7 +304,7 @@ class M2P(object):
         else:
             message_field_dict = get_desc_from_p2p(message=msg)  # type: ignore
         self._parse_msg_desc_method: Optional[str] = parse_msg_desc_method
-        self._field_doc_dict = message_field_dict
+        self._field_doc_dict: Dict[str, DescFromOptionTypedDict] = message_field_dict
         self._default_field = default_field
         self._comment_prefix = comment_prefix
         self._creat_cache: Dict[Union[Type[Message], Descriptor], Type[BaseModel]] = {}
@@ -305,34 +326,27 @@ class M2P(object):
     def model(self) -> Type[BaseModel]:
         return self._gen_model
 
-    def _one_of_handle(self, descriptor: Descriptor) -> Dict[str, OneOfTypedDict]:
-        one_of_dict: Dict[str, OneOfTypedDict] = {}
+    def _one_of_handle(self, descriptor: Descriptor) -> Dict[str, "OneOfTypedDict"]:
+        desc_dict: "DescFromOptionTypedDict" = self._field_doc_dict.get(descriptor.name, {})  # type: ignore
+        one_of_dict: Dict[str, "OneOfTypedDict"] = {}
         for one_of in descriptor.oneofs:
-            if one_of.full_name not in one_of_dict:
-                one_of_dict[one_of.full_name] = {"required": False, "fields": set()}
-            if one_of.full_name in self._field_doc_dict:
+            column_name: str = one_of.full_name
+            if column_name not in one_of_dict:
+                one_of_dict[column_name] = {"required": False, "fields": set()}
+            if desc_dict and column_name in desc_dict["one_of"]:
                 # only PGV or P2P support
-
-                # TODO The use of full name here and the use of name elsewhere will lead to confusion in the storage
-                #  range of field_doc_dict, which must be changed later.
-                one_of_dict[one_of.full_name]["required"] = self._field_doc_dict[one_of.full_name].get(  # type: ignore
-                    "required", False
-                )
+                one_of_dict[column_name]["required"] = desc_dict["one_of"][column_name].get("required", False)
             for _field in one_of.fields:
-                one_of_dict[one_of.full_name]["fields"].add(_field.name)
+                one_of_dict[column_name]["fields"].add(_field.name)
         return one_of_dict
 
-    def _get_pydantic_base(self, pydantic_model_config_dict: Dict[str, Any]) -> Type[BaseModel]:
-        if pydantic_model_config_dict:
+    def _get_pydantic_base(self, config_dict: Dict[str, Any]) -> Type[BaseModel]:
+        if config_dict:
             # Changing the configuration of Config by inheritance
             pydantic_base: Type[BaseModel] = type(  # type: ignore
                 self._pydantic_base.__name__,
                 (self._pydantic_base,),
-                {
-                    "Config": type(
-                        self._pydantic_base.Config.__name__, (self._pydantic_base.Config,), pydantic_model_config_dict
-                    )
-                },
+                {"Config": type(self._pydantic_base.Config.__name__, (self._pydantic_base.Config,), config_dict)},
             )
         else:
             pydantic_base = self._pydantic_base
@@ -346,7 +360,7 @@ class M2P(object):
         annotation_dict: Dict[str, Tuple[Type, Any]] = {}
         validators: Dict[str, classmethod] = {}
         pydantic_model_config_dict: Dict[str, Any] = {}
-        one_of_dict: Dict[str, OneOfTypedDict] = self._one_of_handle(descriptor)
+        one_of_dict: Dict[str, "OneOfTypedDict"] = self._one_of_handle(descriptor)
         if one_of_dict:
             validators["one_of_validator"] = root_validator(pre=True, allow_reuse=True)(check_one_of)
 
@@ -379,24 +393,23 @@ class M2P(object):
                 if column.message_type.name in self._message_type_dict_by_type_name:
                     type_ = self._message_type_dict_by_type_name[column.message_type.name]
                     if column.message_type.name in self._message_default_factory_dict_by_type_name:
+                        # Default factory has a higher priority
                         default_factory = self._message_default_factory_dict_by_type_name[column.message_type.name]
                 elif column.message_type.name.endswith("Entry"):
                     # support google.protobuf.MapEntry
-                    key, value = column.message_type.fields
+                    # key, value = column.message_type.fields
 
-                    if not key.message_type:
-                        key_type: Any = type_dict[key.type]
-                    elif key.message_type.name in self._message_type_dict_by_type_name:
-                        key_type = self._message_type_dict_by_type_name[key.message_type.name]
-                    else:
-                        key_type = self._parse_msg_to_pydantic_model(descriptor=key.message_type)
-                    if not value.message_type:
-                        value_type: Any = type_dict[value.type]
-                    elif value.message_type.name in self._message_type_dict_by_type_name:
-                        value_type = self._message_type_dict_by_type_name[value.message_type.name]
-                    else:
-                        value_type = self._parse_msg_to_pydantic_model(descriptor=value.message_type)
-                    type_ = Dict[key_type, value_type]
+                    dict_type_param_list = []
+                    for k_v_field in column.message_type.fields:
+                        if not k_v_field.message_type:
+                            k_v_type: Any = type_dict[k_v_field.type]
+                        elif k_v_field.message_type.name in self._message_type_dict_by_type_name:
+                            k_v_type = self._message_type_dict_by_type_name[k_v_field.message_type.name]
+                        else:
+                            k_v_type = self._parse_msg_to_pydantic_model(descriptor=k_v_field.message_type)
+                        dict_type_param_list.append(k_v_type)
+
+                    type_ = Dict[tuple(dict_type_param_list)]  # type: ignore
                     default_factory = dict
                 else:
                     # support google.protobuf.Message
@@ -407,18 +420,22 @@ class M2P(object):
                         _class_name: str = column.message_type.name
                         if not is_same_pkg:
                             _class_name = replace_file_name_to_class_name(column.message_type.file.name) + _class_name
-                        type_ = self._parse_msg_to_pydantic_model(
-                            descriptor=column.message_type, class_name=_class_name
-                        )
-                        if not is_same_pkg:
-                            setattr(
-                                type_,
-                                "__doc__",
-                                (
-                                    "Note: The current class does not belong to the package\n"
-                                    f"{_class_name} protobuf path:{column.message_type.file.name}"
-                                ),
+                            type_ = self._parse_msg_to_pydantic_model(
+                                descriptor=column.message_type, class_name=_class_name
                             )
+                            _class_doc: str = (
+                                "Note: The current class does not belong to the package\n"
+                                f"{_class_name} protobuf path:{column.message_type.file.name}"
+                            )
+                            setattr(type_, "__doc__", _class_doc)
+                        else:
+                            if column.message_type.full_name == descriptor.full_name:
+                                # if self-referencing, need use Python type hints postponed annotations
+                                type_ = f'"{_class_name}"'
+                            else:
+                                type_ = self._parse_msg_to_pydantic_model(
+                                    descriptor=column.message_type, class_name=_class_name
+                                )
             elif column.type == FieldDescriptor.TYPE_ENUM:
                 # support google.protobuf.Enum
                 default = 0
@@ -427,14 +444,14 @@ class M2P(object):
                 else:
                     enum_class_dict = {v.name: v.number for v in column.enum_type.values}
                     _class_name = column.enum_type.name
+                    _class_doc = ""
                     if descriptor.file.name != column.enum_type.file.name:
                         _class_name = replace_file_name_to_class_name(column.enum_type.file.name) + _class_name
-                        enum_class_dict["__doc__"] = (
+                        _class_doc = (
                             "Note: The current class does not belong to the package\n"
                             f"{_class_name} protobuf path:{column.enum_type.file.name}"
                         )
-                    else:
-                        enum_class_dict["__doc__"] = ""
+                    enum_class_dict["__doc__"] = _class_doc
                     type_ = IntEnum(_class_name, enum_class_dict)  # type: ignore
             else:
                 if column.label == FieldDescriptor.LABEL_REQUIRED:
@@ -445,6 +462,7 @@ class M2P(object):
             if column.label == FieldDescriptor.LABEL_REPEATED:
                 # support google.protobuf.array
                 if not (column.message_type and column.message_type.name.endswith("Entry")):
+                    # I didn't know that Protobuf's Design of Maps and Lists would be so weird
                     type_ = List[type_]  # type: ignore
                     default_factory = list
                     # TODO support lambda
@@ -452,19 +470,13 @@ class M2P(object):
                         default = Undefined
 
             field = self._default_field
-            field_doc: Union[str, dict] = self._get_field_doc_by_full_name(column.full_name)
-            if field_doc:
-                # Refine field properties with data from desc
-                if isinstance(field_doc, str):
-                    # support protobuf optional by comment
-                    field_doc_dict = self._gen_dict_from_desc_str(field_doc)
-                else:
-                    field_doc_dict = field_doc
+            field_doc_dict = self._get_field_info_dict_by_full_name(column.full_name)
+            if field_doc_dict is not None:
                 if self._parse_msg_desc_method != "PGV":
                     # pgv method not support template var
                     field_doc_dict = self._desc_template.handle_template_var(field_doc_dict)
 
-                field_param_dict: dict = MessagePaitModel(**field_doc_dict).dict()
+                field_param_dict: dict = MessagePaitModel(**field_doc_dict).dict()  # type: ignore
                 # Nested types do not include the `enable`, `field` and `validator`  attributes
                 if not field_param_dict.pop("enable"):
                     continue
@@ -484,17 +496,15 @@ class M2P(object):
                 if field_type:
                     type_ = field_type
                 elif map_type_dict and type_._name == "Dict":
-                    raw_keys_type, raw_values_type = type_.__args__
-                    if "keys" in map_type_dict:
-                        new_keys_type = map_type_dict["keys"]
-                        if issubclass(new_keys_type, raw_keys_type) or raw_keys_type is datetime.datetime:
-                            raw_keys_type = new_keys_type
-                    if "values" in map_type_dict:
-                        new_values_type = map_type_dict["values"]
-                        if issubclass(new_values_type, raw_values_type) or raw_values_type is datetime.datetime:
-                            raw_values_type = new_values_type
-
-                    type_ = Dict[raw_keys_type, raw_values_type]  # type: ignore
+                    new_args_list: List = list(type_.__args__)
+                    for index, k_v_column in enumerate(["keys", "values"]):
+                        raw_k_v_type = new_args_list[index]
+                        if k_v_column not in map_type_dict:
+                            continue
+                        new_k_v_type = map_type_dict[k_v_column]
+                        if issubclass(new_k_v_type, raw_k_v_type) or raw_k_v_type is datetime.datetime:
+                            new_args_list[index] = new_k_v_type
+                    type_ = Dict[tuple(new_args_list)]  # type: ignore
             else:
                 field_param_dict = {"default": default, "default_factory": default_factory}
             use_field = field(**field_param_dict)  # type: ignore
@@ -527,15 +537,20 @@ class M2P(object):
             pait_dict.update(json.loads(line))
         return pait_dict
 
-    def _get_field_doc_by_full_name(self, full_name: str) -> Any:
-        field_doc_dict: dict = self._field_doc_dict
-        key_list = full_name.split(".")[1:]  # ignore package name
+    def _get_field_info_dict_by_full_name(self, full_name: str) -> Optional["FieldInfoTypedDict"]:
+        message_name, *key_list = full_name.split(".")[1:]  # ignore package name
+        if message_name not in self._field_doc_dict:
+            return None
+        desc_dict: "DescFromOptionTypedDict" = self._field_doc_dict[message_name]
+
         for key in key_list:
-            if key in field_doc_dict:
-                field_doc_dict = field_doc_dict[key]
+            if key in desc_dict["message"]:
+                return desc_dict["message"][key]
+            elif key in desc_dict["nested"]:
+                desc_dict = desc_dict["nested"][key]
             else:
                 return None
-        return field_doc_dict
+        return None
 
 
 def msg_to_pydantic_model(
@@ -555,12 +570,21 @@ def msg_to_pydantic_model(
     :param msg: grpc Message or descriptor
     :param default_field: gen pydantic_model default Field, apply only to the outermost pydantic model
     :param comment_prefix: Customize the prefixes that need to be parsed for comments
-    :param parse_msg_desc_method: Define the type of comment to be parsed, if the value is a protobuf file path,
-        it will be parsed by protobuf file; if it is a module of message object, it will be parsed by pyi file
+    :param parse_msg_desc_method:
+        Define a method for extracting the message extension property
+        1.If the value is 'ignore', it means that no extraction is made
+        2.If the value is the Protobuf file path, the Protobuf file is parsed and the information is extracted from
+         the comments in the file
+         Note: The extracted content is a text comment in the Protobuf file
+        3.If the value is a Message object's module, it is extracted from the corresponding pyi file
+         (pyi file is generated by mypy-protobuf)
+         Note: The extracted content is a text comment in the Protobuf file
+        4.If the value is PGV, the corresponding PGV information is extracted from the Message object
+        5.If the value is None (default), the P2P information is extracted from the Message)
     :param local_dict: The variables corresponding to the p2p@local template
     :param pydantic_base: custom pydantic.BaseModel
     :param pydantic_module: custom create model's module name
-    :param desc_template: Template object, which can extend and modify template adaptation rules through inheritance
+    :param desc_template: DescTemplate object, which can extend and modify template adaptation rules through inheritance
     :param message_type_dict_by_type_name: Define the Python type mapping corresponding to each Protobuf Type
     :param message_default_factory_dict_by_type_name: Define the default_factory corresponding to each Protobuf Type
     """
