@@ -15,6 +15,7 @@ try:
 except ModuleNotFoundError:
     from google._upb._message import RepeatedCompositeContainer, RepeatedScalarContainer  # type: ignore
 
+from google.protobuf.internal.containers import RepeatedCompositeFieldContainer, RepeatedScalarFieldContainer
 from google.protobuf.timestamp_pb2 import Timestamp  # type: ignore
 
 __all__ = [
@@ -31,4 +32,13 @@ __all__ = [
     "MessageToDict",
     "RepeatedCompositeContainer",
     "RepeatedScalarContainer",
+    "RepeatedScalarFieldContainer",
+    "RepeatedCompositeFieldContainer",
+    "ProtobufRepeatedType",
+]
+ProtobufRepeatedType = [
+    RepeatedScalarFieldContainer,
+    RepeatedCompositeFieldContainer,
+    RepeatedScalarContainer,
+    RepeatedCompositeContainer,
 ]
