@@ -26,53 +26,62 @@ from protobuf_to_pydantic.grpc_types import (
     FieldDescriptorProto,
     FileDescriptorProto,
 )
-from protobuf_to_pydantic.plugin.types import ProtobufTypeModel
+from protobuf_to_pydantic.plugin.my_types import ProtobufTypeModel
 
 if TYPE_CHECKING:
-    from protobuf_to_pydantic.plugin.config import Config
+    from protobuf_to_pydantic.plugin.config import ConfigModel
 
 logger: logging.Logger = logging.getLogger(__name__)
 
 
 class FileDescriptorProtoToCode(BaseP2C):
-    def __init__(self, fd: FileDescriptorProto, descriptors: Descriptors, config: "Config"):
+    def __init__(self, fd: FileDescriptorProto, descriptors: Descriptors, config: "ConfigModel"):
+        # Prevent mutable variables from being shared with other configs
+        config = config.copy(deep=True)
         super().__init__(
             customer_import_set=config.customer_import_set,
             customer_deque=config.customer_deque,
             module_path=config.module_path,
             code_indent=config.code_indent,
+            pyproject_file_path=config.pyproject_file_path,
         )
+        self.config = config
         self._fd: FileDescriptorProto = fd
         self._descriptors: Descriptors = descriptors
-        self._desc_template: DescTemplate = config.desc_template
+        self._desc_template: DescTemplate = config.desc_template_instance
 
         if config.base_model_class is BaseModel:
             self._import_set.add("from pydantic import BaseModel")
         else:
             self._add_import_code(config.base_model_class.__module__, config.base_model_class.__name__)
+        self._parse_desc_name_dict: Dict[str, str] = {}
         self._parse_field_descriptor()
 
     def _add_other_module_pkg(self, other_fd: FileDescriptorProto, type_str: str) -> None:
-        if other_fd.name != self._fd.name:
-            # Generate the corresponding import statement
-            # e.g:
-            #   fd name:example_proto/demo/demo.proto
-            #   other_fd name: example_proto/common/single.proto
-            #   output: from ..common.single_p2p import DemoMessage
-            fd_path_list: Tuple[str, ...] = Path(self._fd.name).parts
-            message_path_list: Tuple[str, ...] = Path(other_fd.name).parts
-            index: int = -1
-            for _index in range(min(len(fd_path_list), len(message_path_list))):
-                if message_path_list[_index] == fd_path_list[_index]:
-                    index = _index
+        if other_fd.name == self._fd.name:
+            return
+        # Generate the corresponding import statement
+        # e.g:
+        #   fd name:example_proto/demo/demo.proto
+        #   other_fd name: example_proto/common/single.proto
+        #   output: from ..common.single_p2p import DemoMessage
+        fd_path_list: Tuple[str, ...] = Path(self._fd.name).parts
+        message_path_list: Tuple[str, ...] = Path(other_fd.name).parts
+        index: int = -1
+        for _index in range(min(len(fd_path_list), len(message_path_list))):
+            if message_path_list[_index] == fd_path_list[_index]:
+                index = _index
 
-            module_name: str = (
-                ".".join(message_path_list[index + 1 : -1]) + "." + message_path_list[-1].replace(".proto", "") + "_p2p"
-            )
-            logger.info((self._fd.name, other_fd.name, index))
-            if index != "-1":
-                module_name = "." * (len(message_path_list) - (index + 1)) + module_name
-            self._add_import_code(module_name, type_str)
+        module_name: str = (
+            ".".join(message_path_list[index + 1 : -1])
+            + "."
+            + message_path_list[-1].replace(".proto", "")
+            + self.config.file_name_suffix
+        )
+        logger.info((self._fd.name, other_fd.name, index))
+        if index != "-1":
+            module_name = "." * (len(message_path_list) - (index + 1)) + module_name
+        self._add_import_code(module_name, type_str)
 
     def _enum(self, enums: Iterable[EnumDescriptorProto], scl_prefix: SourceCodeLocation, indent: int = 0) -> str:
         """
@@ -123,6 +132,7 @@ class FileDescriptorProtoToCode(BaseP2C):
     # flake8: noqa: C901
     def _message_field_handle(
         self,
+        desc: DescriptorProto,
         field: FieldDescriptorProto,
         indent: int,
         nested_message_config_dict: dict,
@@ -158,6 +168,12 @@ class FileDescriptorProtoToCode(BaseP2C):
 
                 message_fd: FileDescriptorProto = self._descriptors.message_to_fd[field.type_name]
                 self._add_other_module_pkg(message_fd, type_str)
+                if message == desc:
+                    # if self-referencing, need use Python type hints postponed annotations
+                    type_str = f'"{type_str}"'
+                elif message_fd.name == self._fd.name and message.name not in {i.name for i in desc.nested_type}:
+                    # If the referenced Message is generated later, it needs to be generated in advance
+                    self._content_deque.append(self._message(message, [FileDescriptorProto.ENUM_TYPE_FIELD_NUMBER]))
 
         elif field.type == 14:
             # enum handle
@@ -184,7 +200,7 @@ class FileDescriptorProtoToCode(BaseP2C):
             rule_type_str = "repeated"
         if len(field.options.ListFields()) != 0 and rule_type_str and not skip_validate_rule:
             # protobuf option support
-            field_option_info_dict = field_option_handle(rule_type_str, field.type_name, field)
+            field_option_info_dict: dict = field_option_handle(rule_type_str, field.type_name, field)  # type: ignore
 
             skip = field_option_info_dict.pop("skip", False)
             if nested_message_name:
@@ -297,6 +313,8 @@ class FileDescriptorProtoToCode(BaseP2C):
         self._add_import_code("google.protobuf.message", "Message")
         content: str = ""
         class_name = desc.name if desc.name not in PYTHON_RESERVED else "_r_" + desc.name
+        if class_name in self._parse_desc_name_dict:
+            return self._parse_desc_name_dict[class_name]
         class_content = " " * indent + f"class {class_name}(BaseModel):\n"
         class_head_content = ""
         class_field_content = ""
@@ -310,7 +328,7 @@ class FileDescriptorProtoToCode(BaseP2C):
                 use_custom_type = True
 
             _content_tuple: Optional[Tuple[str, str]] = self._message_field_handle(
-                field, indent, nested_message_config_dict, skip_validate_rule=skip_validate_rule
+                desc, field, indent, nested_message_config_dict, skip_validate_rule=skip_validate_rule
             )
             if _content_tuple:
                 class_head_content += _content_tuple[0]
@@ -340,17 +358,18 @@ class FileDescriptorProtoToCode(BaseP2C):
             class_head_content = config_content + class_head_content
         content += "\n".join([i for i in [class_content, class_head_content, class_field_content] if i])
         content += "\n" if indent > 0 else "\n\n"
+        self._parse_desc_name_dict[class_name] = content
         return content
 
     def _get_protobuf_type_model(self, field: FieldDescriptorProto) -> ProtobufTypeModel:
-        rule_type_str: Optional[str] = None
+        rule_type_str: str = ""
         type_factory: Optional[Any] = None
         if field.type in type_dict:
             type_factory = type_dict[field.type]
             return ProtobufTypeModel(
                 type_factory=type_factory,
                 py_type_str=self._get_value_code(type_factory),
-                rule_type_str=protobuf_common_type_dict.get(field.type, None),
+                rule_type_str=protobuf_common_type_dict.get(field.type, ""),
             )
         elif field.type_name.startswith(".google.protobuf"):
             _type_str = field.type_name.split(".")[-1]
@@ -373,9 +392,16 @@ class FileDescriptorProtoToCode(BaseP2C):
                 rule_type_str = "any"
                 type_factory = AnyMessage
                 self._add_import_code("google.protobuf.any_pb2", "Any")
+            elif field.type_name.split(".")[-2] == "Struct":
+                py_type_str = "Dict"
+                rule_type_str = "struct"
+                type_factory = dict
             else:
                 logger.error(f"Not support type {field.type_name}")
-                py_type_str = ""
+                py_type_str = "Any"
+                rule_type_str = "any"
+                type_factory = AnyMessage
+                self._add_import_code("google.protobuf.any_pb2", "Any")
             return ProtobufTypeModel(
                 type_factory=type_factory,
                 rule_type_str=rule_type_str,
